unit Forms.SynopticUnit;

interface

uses
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  System.UIConsts,System.Math.Vectors,System.math,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.TMSFNCTypes, FMX.TMSFNCUtils, FMX.TMSFNCGraphics, FMX.TMSFNCGraphicsTypes,
  FMX.TMSFNCStatusBar, FMX.TMSFNCCustomControl, FMX.TMSFNCTreeViewBase,
  FMX.TMSFNCTreeViewData, FMX.TMSFNCCustomTreeView, FMX.TMSFNCTreeView,
  FMX.TMSFNCObjectInspector, FMX.Objects, FMX.TMSFNCPageControl,
  FMX.TMSFNCTabSet, FMX.Controls.Presentation, FMX.StdCtrls, FMX.TMSFNCTrackBar,
  FMX.Layouts, VarSynopticTypeUnit, IdBaseComponent, IdCoder, IdCoder3to4,
  IdCoderMIME;

type
  TSynopticForm = class(TForm)
    StatusBar: TTMSFNCStatusBar;
    Panel1: TPanel;
    Panel2: TPanel;
    ObjectInspector: TTMSFNCObjectInspector;
    LayoutMain: TLayout;
    Splitter1: TSplitter;
    IdDecoderMIME1: TIdDecoderMIME;
    MyPanel: TPanel;
    DrawingBackground: TRectangle;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure RefreshSynoptic(sender: TObject);
    procedure ScrollBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure ScrollBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Single);
    procedure ScrollBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
    procedure MyPanelPaint(Sender: TObject; Canvas: TCanvas;
      const ARect: TRectF);
    procedure Panel3MouseLeave(Sender: TObject);
    procedure FormMouseWheel(Sender: TObject; Shift: TShiftState;
      WheelDelta: Integer; var Handled: Boolean);
    procedure MyPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Single);
  private
    { Dclarations prives }
    FLastMx    :Single;
    FLastMy    :Single;
    FMouseDown :Boolean;
    FShowGrid  :Boolean;
    FDoSnap    :Boolean;
    FDoPan     :Boolean;
    FCentered  :Boolean;
    procedure setDoPan(Value:boolean);
    procedure setDoSnap(Value:boolean);
    procedure setShowGrid(Value:boolean);

  public
    { Dclarations publiques }
    Synoptic:TVKQVSynopticType;
    procedure LoadSynopticFile(FileName:string);
    procedure Pan(dx,dy:real);
    procedure CenterView;
    procedure SetOrigin;
    procedure Zoom(factor:single);
    procedure SetGridSize(value:integer);
    procedure SetSnapSize(value:integer);
  published
    property DoPan:Boolean read FDoPan write setDoPan;
    property DoSnap:Boolean read FDoSnap write setDoSnap;
    property ShowGrid:Boolean read FShowGrid write setShowGrid;
  end;

var
  SynopticForm: TSynopticForm;

implementation

{$R *.fmx}

procedure TSynopticForm.FormCreate(Sender: TObject);
begin
 Synoptic:=TVKQVSynopticType.Create;
 Synoptic.OnRefresh:=RefreshSynoptic;
 FCentered:=true;
end;

procedure TSynopticForm.FormDestroy(Sender: TObject);
begin
 Synoptic.Free;
end;

procedure TSynopticForm.FormMouseWheel(Sender: TObject; Shift: TShiftState;
  WheelDelta: Integer; var Handled: Boolean);
var
 scale:single;
begin
  scale:=MyPanel.Scale.X+(wheelDelta)*0.001;

  if scale <0.1 then scale:=0.1;

  MyPanel.Scale.X:=scale;
  Mypanel.Scale.Y:=scale;
  CenterView;
  Handled:=true;
end;


procedure TSynopticForm.LoadSynopticFile(FileName: string);
begin
  Synoptic.OpenSynopticFile(FileName);
  ObjectInspector.&Object:=Synoptic;
  MyPanel.Width:=Synoptic.Width;
  MyPanel.Height:=Synoptic.Height;
  MyPanel.Position.X:=0;
  MyPanel.Position.Y:=0;
end;

procedure TSynopticForm.ScrollBox1MouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Single);
begin
 FLastMx:=X;
 FLastMy:=Y;
 FMouseDown:=true;
end;

procedure TSynopticForm.ScrollBox1MouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Single);
var
 dx,dy:single;
begin
  if (FMouseDown)and(FDoPan) then
  begin
   dx:=FLastMx-x;
   dy:=FLastMy-y;
   Pan(dx,dy);
   FLastMx:=x;
   FLastMy:=y;
  end;
end;

procedure TSynopticForm.ScrollBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Single);
begin
 FMouseDown:=false;
end;

procedure TSynopticForm.setDoPan(Value: boolean);
begin
 if FdoPan<>Value then
 begin
   FdoPan:=not FDoPan;
   MyPanel.Enabled:=not FDoPan;
   if Fdopan then DrawingBackground.Cursor:=crSizeAll else DrawingBackground.Cursor:=crDefault;

 end;
end;

procedure TSynopticForm.setDoSnap(Value: boolean);
begin
 if FDoSnap<>value then
 begin
   FDoSnap:=value;
 end;
end;



procedure TSynopticForm.SetGridSize(value: integer);
begin
  Synoptic.GridSize:=value;
  ObjectInspector.RebuildList;
end;

procedure TSynopticForm.SetSnapSize(value: integer);
begin
  Synoptic.SnapSize:=value;
  ObjectInspector.RebuildList;
end;


procedure TSynopticForm.SetOrigin;
begin
  Mypanel.Scale.X:=1;
  MyPanel.Scale.Y:=1;
  FCentered:=true;
  Centerview;
  MyPanel.Repaint;
end;

procedure TSynopticForm.setShowGrid(Value: boolean);
begin
 if FshowGrid<>value then
 begin
   FShowGrid:=value;
   MyPanel.repaint;
 end;
end;



procedure TSynopticForm.Zoom(factor: single);
begin
 MyPanel.Scale.X:=MyPanel.Scale.X+factor;
 if MyPanel.Scale.X<0.1 then MyPanel.Scale.X:=0.1;

 MyPanel.Scale.Y:=MyPanel.Scale.Y+factor;
 if myPanel.Scale.Y<0.1 then MyPanel.Scale.Y:=0.1;

 Centerview;
end;

procedure TSynopticForm.CenterView;
begin
 if FCentered then
 begin
   Mypanel.Position.X:=(DrawingBackground.Width / 2)-((MyPanel.Width*MyPanel.Scale.X)/2);
   MyPanel.Position.Y:=(DrawingBackground.Height / 2)-((MyPanel.Height*MyPanel.Scale.Y)/2);
 end;
end;




procedure TSynopticForm.Pan(dx, dy: real);
begin
 MyPanel.Position.X:=MyPanel.Position.X-dx;
 MyPanel.Position.Y:=MyPanel.Position.Y-dy;
 FCentered:=false;
end;

procedure TSynopticForm.Panel3MouseLeave(Sender: TObject);
begin
 FMouseDown:=false;
end;

procedure TSynopticForm.MyPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Single);
begin
  StatusBar.Panels.Items[0].text:=format('X: %1.0f Y: %1.0f',[x,y]);
end;

procedure TSynopticForm.MyPanelPaint(Sender: TObject; Canvas: TCanvas;
  const ARect: TRectF);
var
 i,x,y:integer;
begin
  if Canvas.BeginScene then
  begin
   try


   finally
     Canvas.EndScene;
   end;
  end;

end;

procedure TSynopticForm.RefreshSynoptic(sender: TObject);
begin
  if MyPanel.Width<>Synoptic.Width then MyPanel.Width:=Synoptic.Width;
  if MyPanel.Height<>Synoptic.Height then MyPanel.Height:=Synoptic.Height;
  CenterView;

  MyPanel.Repaint;
end;

end.
