unit VarSynopticTypeUnit;

interface

uses System.Inifiles,
     System.IOUtils,
     System.SysUtils,
     System.Json,
     System.Classes,
     System.Generics.Defaults,
     System.Generics.Collections,
     System.UITypes,
     System.UIConsts;

Type

 // Eventhandler  appeler lorsque le UI doit tre rafrachit.
 TRefreshEvent      = procedure (sender:TObject) of object;

// Dfinition des variables d'un synoptique

 TVKQVSynopticWindowType = record
   Caption               :string;
   BackGroundImageFile   :string;
   BackGroundImage       :string;
   ScriptFile            :string;
   Script                :string;
   LogicFile             :string;
   Logic                 :string;
   Color                 :string;
   Left,Top,Height,Width :integer;
   WindowHeight          :integer;
   WindowWidth           :integer;
   HideTitle             :boolean;
   ShowAtPosition        :boolean;
   DoClose               :boolean;
   DockToMain            :boolean;
   Scrollable            :boolean;
   Sizeable              :boolean;
   Minimize              :boolean;
   Maximize              :boolean;
   FitInWindow           :boolean;
   KeepAspect            :boolean;
   Zoomable              :boolean;
   AntiAliasing          :boolean;
   RefreshRate           :integer;
   GraphicFilter         :integer;
   Index                 :array[1..3] of integer;
   HorizGridEnable       :boolean;
   VertiGridEnable       :boolean;
   HorizGridSize         :integer;
   VertiGridSize         :integer;
   HorizSnapSize         :integer;
   VertiSnapSize         :integer;
   ConsoleVisible        :boolean;
   ConsoleHeight         :integer;
 end;



 // Variable correspondant aux donnes d'un synoptique
 TVKQVSynopticType = Class
   private
     FWindow      :TVKQVSynopticWindowType;

     FFileName    :string;
     FRefresh     :TRefreshEvent;
     function      GetBackgroundColor:TAlphaColor;
     procedure     SetBackGroundColor(Value:TAlphaColor);
     procedure     SetWidth(Value:Integer);
     procedure     SetHeight(Value:Integer);
     procedure     SetGridSize(Value:Integer);
     procedure     SetSnapSize(Value:Integer);
   public
     constructor   Create; overload;
     destructor    Destroy; override;
     procedure     Clear;
     procedure     OpenSynopticFile(TheFile:string);
   published
     property      Caption:string Read Fwindow.caption;
     property      BackgroundColor:TAlphaColor read GetBackgroundColor write SetBackgroundColor;
     property      BackGroundImage:string read FWindow.BackgroundImage;
     property      Width:integer read FWindow.Width write SetWidth;
     property      Height:integer read FWindow.Height write SetHeight;
     property      GridSize:integer read FWindow.HorizGridSize write SetGridSize;
     property      SnapSize:integer read FWindow.HorizSnapSize write SetSnapSize;
     property      FileName:string read FFileName;
     property      OnRefresh:TRefreshEvent read FRefresh write FRefresh;
 End;


implementation



//TVKQVSynopticType
Constructor TVKQVSynopticType.Create;
begin

  inherited;
end;

Destructor  TVKQVSynopticType.Destroy;
begin

  inherited;
end;

function TVKQVSynopticType.GetBackgroundColor: TAlphaColor;
begin
  result:=stringToAlphaColor(FWindow.Color);
  if assigned(FRefresh) then FRefresh(self);
end;



Procedure   TVKQVSynopticType.Clear;
begin
  fillchar(FWindow,sizeof(TVKQVSynopticWindowType),0);
end;

Procedure   TVKQVSynopticType.OpenSynopticFile(TheFile: string);
begin
  FFileName:=TheFile;
end;

procedure TVKQVSynopticType.SetBackGroundColor(Value: TAlphaColor);
begin
 FWindow.Color:=AlphaColorToString(Value);
 if assigned(onRefresh) then OnRefresh(self);
end;


procedure TVKQVSynopticType.SetGridSize(Value: Integer);
begin
 if FWindow.HorizGridSize<>Value then
 begin
   if Value>0 then FWindow.HorizGridSize:=Value
              else FWindow.HorizGridSize:=5;
   if assigned(onRefresh) then OnRefresh(self);
 end;
end;

procedure TVKQVSynopticType.SetSnapSize(Value: Integer);
begin
 if FWindow.HorizSnapSize<>Value then
 begin
   if Value>0 then FWindow.HorizSnapSize:=Value
              else FWindow.HorizSnapSize:=5;
   if assigned(onRefresh) then OnRefresh(self);
 end;
end;

procedure TVKQVSynopticType.SetHeight(Value: Integer);
begin
  if FWindow.Height<>value then
  begin
    if value>0 then FWindow.Height:=value
               else FWindow.Height:=1;
    if assigned(onRefresh) then OnRefresh(self);

  end;
end;

procedure TVKQVSynopticType.SetWidth(Value: Integer);
begin
  if FWindow.width<>value then
  begin
    if value>0 then FWindow.width:=value
               else FWindow.width:=1;
    if assigned(onRefresh) then OnRefresh(self);
  end;
end;


end.
